source("RemembR/R/RemembeR.R")
library(arrow)
library(data.table)
library(ggplot2)
library(filelock)
library(argparser)

parser <- arg_parser("Simulate data and fit corrected models.")
parser <- add_argument(parser, "--infile", default="", help="name of the file to read.")
parser <- add_argument(parser, "--name", default="", help="The name to safe the data to in the remember file.")
args <- parse_args(parser)

build_plot_dataset <- function(df){
    x.naive <- df[,.(N, m, Bxy, Bxy.est.naive, Bxy.ci.lower.naive, Bxy.ci.upper.naive)]
    x.naive <- x.naive[,':='(true.in.ci = as.integer((Bxy >= Bxy.ci.lower.naive) & (Bxy <= Bxy.ci.upper.naive)),
                             zero.in.ci = (0 >= Bxy.ci.lower.naive) & (0 <= Bxy.ci.upper.naive),
                             bias = Bxy - Bxy.est.naive,
                             Bxy.est.naive = Bxy.est.naive,
                             sign.correct = as.integer(sign(Bxy) == sign(Bxy.est.naive)))]

    x.naive.plot <- x.naive[,.(p.true.in.ci = mean(true.in.ci),
                               mean.bias = mean(bias),
                               mean.est = mean(Bxy.est.naive),
                               var.est = var(Bxy.est.naive),
                               N.sims = .N,
                               p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                               variable='x',
                               method='Naive'
                               ),
                            by=c('N','m')]
    

    g.naive <- df[,.(N, m, Bgy, Bgy.est.naive, Bgy.ci.lower.naive, Bgy.ci.upper.naive)]
    g.naive <- g.naive[,':='(true.in.ci = as.integer((Bgy >= Bgy.ci.lower.naive) & (Bgy <= Bgy.ci.upper.naive)),
                             zero.in.ci = (0 >= Bgy.ci.lower.naive) & (0 <= Bgy.ci.upper.naive),
                             bias = Bgy - Bgy.est.naive,
                             Bgy.est.naive = Bgy.est.naive,
                             sign.correct = as.integer(sign(Bgy) == sign(Bgy.est.naive)))]

    g.naive.plot <- g.naive[,.(p.true.in.ci = mean(true.in.ci),
                               mean.bias = mean(bias),
                               mean.est = mean(Bgy.est.naive),
                               var.est = var(Bgy.est.naive),
                               N.sims = .N,
                               p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                               variable='g',
                               method='Naive'
                               ),
                            by=c('N','m')]
    

    x.feasible <- df[,.(N, m, Bxy, Bxy.est.feasible, Bxy.ci.lower.feasible, Bxy.ci.upper.feasible)]
    x.feasible <- x.feasible[,':='(true.in.ci = as.integer((Bxy >= Bxy.ci.lower.feasible) & (Bxy <= Bxy.ci.upper.feasible)),
                                   zero.in.ci = (0 >= Bxy.ci.lower.feasible) & (0 <= Bxy.ci.upper.feasible),
                                   bias = Bxy - Bxy.est.feasible,
                                   Bxy.est.feasible = Bxy.est.feasible,
                                   sign.correct = as.integer(sign(Bxy) == sign(Bxy.est.feasible)))]

    x.feasible.plot <- x.feasible[,.(p.true.in.ci = mean(true.in.ci),
                                     mean.bias = mean(bias),
                                     mean.est = mean(Bxy.est.feasible),
                                     var.est = var(Bxy.est.feasible),
                                     N.sims = .N,
                                     p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                                     variable='x',
                                     method='Feasible'
                                     ),
                                  by=c('N','m')]
    

    g.feasible <- df[,.(N, m, Bgy, Bgy.est.feasible, Bgy.ci.lower.feasible, Bgy.ci.upper.feasible)]
    g.feasible <- g.feasible[,':='(true.in.ci = as.integer((Bgy >= Bgy.ci.lower.feasible) & (Bgy <= Bgy.ci.upper.feasible)),
                                   zero.in.ci = (0 >= Bgy.ci.lower.feasible) & (0 <= Bgy.ci.upper.feasible),
                                   bias = Bgy - Bgy.est.feasible,
                                   Bgy.est.feasible = Bgy.est.feasible,
                                   sign.correct = as.integer(sign(Bgy) == sign(Bgy.est.feasible)))]

    g.feasible.plot <- g.feasible[,.(p.true.in.ci = mean(true.in.ci),
                                     mean.bias = mean(bias),
                                     mean.est = mean(Bgy.est.feasible),
                                     var.est = var(Bgy.est.feasible),
                                     N.sims = .N,
                                     p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                                     variable='g',
                                     method='Feasible'
                                     ),
                                  by=c('N','m')]
    


    x.amelia.full <- df[,.(N, m, Bxy, Bxy.est.true, Bxy.ci.lower.amelia.full, Bxy.ci.upper.amelia.full, Bxy.est.amelia.full)]

    x.amelia.full <- x.amelia.full[,':='(true.in.ci = (Bxy.est.true >= Bxy.ci.lower.amelia.full) & (Bxy.est.true <= Bxy.ci.upper.amelia.full),
                                         zero.in.ci = (0 >= Bxy.ci.lower.amelia.full) & (0 <= Bxy.ci.upper.amelia.full),
                                         bias = Bxy.est.true - Bxy.est.amelia.full,
                                         sign.correct = sign(Bxy.est.true) == sign(Bxy.est.amelia.full))]

    x.amelia.full.plot <- x.amelia.full[,.(p.true.in.ci = mean(as.integer(true.in.ci)),
                                           mean.bias = mean(bias),
                                           p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                                           mean.est = mean(Bxy.est.amelia.full),
                                           var.est = var(Bxy.est.amelia.full),
                                           N.sims = .N,
                                           variable='x',
                                           method='Multiple imputation'
                                           ),
                                        by=c('N','m')]


    g.amelia.full <- df[,.(N, m, Bgy.est.true, Bgy.est.amelia.full, Bgy.ci.lower.amelia.full, Bgy.ci.upper.amelia.full)]
    g.amelia.full <- g.amelia.full[,':='(true.in.ci = (Bgy.est.true >= Bgy.ci.lower.amelia.full) & (Bgy.est.true <= Bgy.ci.upper.amelia.full),
                                         zero.in.ci = (0 >= Bgy.ci.lower.amelia.full) & (0 <= Bgy.ci.upper.amelia.full),
                                         bias =  Bgy.est.amelia.full - Bgy.est.true,
                                         sign.correct = sign(Bgy.est.true) == sign(Bgy.est.amelia.full))]

    g.amelia.full.plot <- g.amelia.full[,.(p.true.in.ci = mean(as.integer(true.in.ci)),
                                           mean.bias = mean(bias),
                                           p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                                           mean.est = mean(Bgy.est.amelia.full),
                                           var.est = var(Bgy.est.amelia.full),
                                           N.sims = .N,
                                           variable='g',
                                           method='Multiple imputation'
                                           ),
                                        by=c('N','m')]

    x.mle <- df[,.(N,m, Bxy.est.true, Bxy.est.mle, Bxy.ci.lower.mle, Bxy.ci.upper.mle)]

    x.mle <- x.mle[,':='(true.in.ci = (Bxy.est.true >= Bxy.ci.lower.mle) & (Bxy.est.true <= Bxy.ci.upper.mle),
                                         zero.in.ci = (0 >= Bxy.ci.lower.mle) & (0 <= Bxy.ci.upper.mle),
                                         bias =  Bxy.est.mle - Bxy.est.true,
                                         sign.correct = sign(Bxy.est.true) == sign(Bxy.est.mle))]

    x.mle.plot <- x.mle[,.(p.true.in.ci = mean(as.integer(true.in.ci)),
                                   mean.bias = mean(bias),
                                   p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                                   mean.est = mean(Bxy.est.mle),
                                   var.est = var(Bxy.est.mle),
                                   N.sims = .N,
                                   variable='x',
                                   method='Maximum Likelihood'
                                   ),
                                by=c('N','m')]



    g.mle <- df[,.(N,m, Bgy.est.true, Bgy.est.mle, Bgy.ci.lower.mle, Bgy.ci.upper.mle)]

    g.mle <- g.mle[,':='(true.in.ci = (Bgy.est.true >= Bgy.ci.lower.mle) & (Bgy.est.true <= Bgy.ci.upper.mle),
                                         zero.in.ci = (0 >= Bgy.ci.lower.mle) & (0 <= Bgy.ci.upper.mle),
                                         bias =  Bgy.est.mle - Bgy.est.true,
                                         sign.correct = sign(Bgy.est.true) == sign(Bgy.est.mle))]

    g.mle.plot <- g.mle[,.(p.true.in.ci = mean(as.integer(true.in.ci)),
                                   mean.bias = mean(bias),
                                   p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                                   mean.est = mean(Bgy.est.mle),
                                   var.est = var(Bgy.est.mle),
                                   N.sims = .N,
                                   variable='g',
                                   method='Maximum Likelihood'
                                   ),
                                by=c('N','m')]




    x.pseudo <- df[,.(N,m, Bxy.est.true, Bxy.est.pseudo, Bxy.ci.lower.pseudo, Bxy.ci.upper.pseudo)]

    x.pseudo <- x.pseudo[,':='(true.in.ci = (Bxy.est.true >= Bxy.ci.lower.pseudo) & (Bxy.est.true <= Bxy.ci.upper.pseudo),
                                         zero.in.ci = (0 >= Bxy.ci.lower.pseudo) & (0 <= Bxy.ci.upper.pseudo),
                                         bias =  Bxy.est.pseudo - Bxy.est.true,
                                         sign.correct = sign(Bxy.est.true) == sign(Bxy.est.pseudo))]

    x.pseudo.plot <- x.pseudo[,.(p.true.in.ci = mean(as.integer(true.in.ci)),
                                   mean.bias = mean(bias),
                                   p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                                   mean.est = mean(Bxy.est.pseudo),
                                   var.est = var(Bxy.est.pseudo),
                                   N.sims = .N,
                                   variable='x',
                                   method='Pseudo Likelihood'
                                   ),
                                by=c('N','m')]



    g.pseudo <- df[,.(N,m, Bgy.est.true, Bgy.est.pseudo, Bgy.ci.lower.pseudo, Bgy.ci.upper.pseudo)]

    g.pseudo <- g.pseudo[,':='(true.in.ci = (Bgy.est.true >= Bgy.ci.lower.pseudo) & (Bgy.est.true <= Bgy.ci.upper.pseudo),
                                         zero.in.ci = (0 >= Bgy.ci.lower.pseudo) & (0 <= Bgy.ci.upper.pseudo),
                                         bias =  Bgy.est.pseudo - Bgy.est.true,
                                         sign.correct = sign(Bgy.est.true) == sign(Bgy.est.pseudo))]

    g.pseudo.plot <- g.pseudo[,.(p.true.in.ci = mean(as.integer(true.in.ci)),
                                   mean.bias = mean(bias),
                                   p.sign.correct = mean(as.integer(sign.correct & (! zero.in.ci))),
                                   mean.est = mean(Bgy.est.pseudo),
                                   var.est = var(Bgy.est.pseudo),
                                   N.sims = .N,
                                   variable='g',
                                   method='Pseudo Likelihood'
                                   ),
                                by=c('N','m')]


    
    accuracy <- df[,mean(accuracy)]

    plot.df <- rbindlist(list(x.naive.plot,g.naive.plot,x.amelia.full.plot,g.amelia.full.plot,x.mle.plot, g.mle.plot, x.pseudo.plot, g.pseudo.plot, x.feasible.plot, g.feasible.plot),use.names=T)

    plot.df[,accuracy := accuracy]

    plot.df <- plot.df[,":="(sd.est=sqrt(var.est)/N.sims)]

    return(plot.df)
}


df <- read_feather(args$infile)
plot.df <- build_plot_dataset(df)
remember(plot.df,args$name)


## df[gmm.ER_pval<0.05]





## ggplot(plot.df[variable=='x'], aes(y=mean.est, ymax=mean.est + var.est/2, ymin=mean.est-var.est/2, x=method)) + geom_pointrange() + facet_grid(-m~N) + scale_x_discrete(labels=label_wrap_gen(10))

## ggplot(plot.df,aes(y=N,x=m,color=p.sign.correct)) + geom_point() + facet_grid(variable ~ method) + scale_color_viridis_c(option='D') + theme_minimal() + xlab("Number of gold standard labels") + ylab("Total sample size") 

## ggplot(plot.df,aes(y=N,x=m,color=abs(mean.bias))) + geom_point() + facet_grid(variable ~ method) + scale_color_viridis_c(option='D') + theme_minimal() + xlab("Number of gold standard labels") + ylab("Total sample size") 
